@@ -1,19 +1,19 @@
 isotopeAnnotation <- function(df.annotation, anclique) {
-    # Isotopes of grade > 0 are excluded from annotation
-    # This function adds the annotation, in case it exists,
-    # to all isotopes of grade > 0 as they should have the same
-    # annotation than its isotope of grade 0
-    isofeatures <- anclique$isotopes$feature[
-        which(anclique$isotopes$grade == 0)]
+    ## Isotopes of grade > 0 are excluded from annotation
+    ## This function adds the annotation, in case it exists,
+    ## to all isotopes of grade > 0 as they should have the same
+    ## annotation than its isotope of grade 0
+    isofeatures <- anclique@isotopes$feature[
+        which(anclique@isotopes$grade == 0)]
     extraAnList <- lapply(isofeatures, function(x) {
-        cluster <- anclique$isotopes$cluster[
-            which(anclique$isotopes$feature == x)]
-        extraf <- anclique$isotopes$feature[
-            which(anclique$isotopes$cluster == cluster)]
-        #now drop the isotope of grade0 because it already has annotation
+        cluster <- anclique@isotopes$cluster[
+            which(anclique@isotopes$feature == x)]
+        extraf <- anclique@isotopes$feature[
+            which(anclique@isotopes$cluster == cluster)]
+        ## now drop the isotope of grade0 because it already has annotation
         extraf <- extraf[-1*which(extraf == x)]
         an <- df.annotation[which(df.annotation$feature == x),]
-        # annotation for all isotopes should be the same
+        ## annotation for all isotopes should be the same
         extraan <- lapply(extraf, function(x) {
             an$feature = x
             an
@@ -27,16 +27,16 @@ isotopeAnnotation <- function(df.annotation, anclique) {
 
 checkadinfo <- function(adinfo, polarity) {
     sameCols = match(colnames(adinfo),
-                     c("adduct","log10freq","massdiff","nummol","charge"))
+        c("adduct","log10freq","massdiff","nummol","charge"))
     if(sum(!is.na(sameCols)) < 5) {
         stop(paste("some missing colnames, check that column",
-                   "'adduct','log10freq','massdiff','nummol' and 'charge'",
-                   "are included"))
+            "'adduct','log10freq','massdiff','nummol' and 'charge'",
+            "are included"))
     }
     if(polarity == "positive") {
-        # Separate them in adducts with charge >1, 
-        # adducts with nmol >1 && charge ==1, 
-        # and adducts with charge and nmol = 1
+        ## Separate them in adducts with charge >1, 
+        ## adducts with nmol >1 && charge ==1, 
+        ## and adducts with charge and nmol = 1
         chargead = adinfo[which(adinfo$charge > 1),]
         chargead = chargead[order(chargead$massdiff),]
 
@@ -51,9 +51,9 @@ checkadinfo <- function(adinfo, polarity) {
         # trying to virtually sort all adducts from smaller massDiff to bigger
         returnAdinfo = rbind(chargead, normalad, nummolad)
     } else {
-        # Separate them in adducts with charge < -1, 
-        # adducts with nmol >1 && charge == -1
-        # and adducts with charge and nmol = 1
+        ## Separate them in adducts with charge < -1, 
+        ## adducts with nmol >1 && charge == -1
+        ## and adducts with charge and nmol = 1
         chargead = adinfo[which(adinfo$charge < -1),]
         chargead = chargead[order(chargead$massdiff),]
 
@@ -63,9 +63,9 @@ checkadinfo <- function(adinfo, polarity) {
         normalad = adinfo[which(adinfo$charge == -1),]
         normalad = normalad[which(normalad$nummol == 1),]
 
-        # Sort them from adducts with charge >1
-        # normal and adducts with nummol > 1
-        # trying to virtually sort all adducts from smaller massDiff to bigger
+        ## Sort them from adducts with charge >1
+        ## normal and adducts with nummol > 1
+        ## trying to virtually sort all adducts from smaller massDiff to bigger
         returnAdinfo = rbind(chargead, normalad, nummolad)
     }
     return(returnAdinfo)
@@ -73,23 +73,65 @@ checkadinfo <- function(adinfo, polarity) {
 
 
 getIsoCharge <- function(df.clique, iso) {
-    # this function sets charge to features that are isotopes 
-    # because with isotope annotation they already have charge
-    # and therefore, adduct annotation is limited to that charge
-    chargeV <- sapply(1:nrow(df.clique), function(x) {
+    ## this function sets charge to features that are isotopes 
+    ## because with isotope annotation they already have charge
+    ## and therefore, adduct annotation is limited to that charge
+    chargeV <- vapply(seq_len(nrow(df.clique)), function(x) {
         charge <- 0
         if( length(grep("[",df.clique[x,"isotope"],
-                        fixed = T)) != 0 ) {
+            fixed = TRUE)) != 0 ) {
             feat <- df.clique[x,"feature"]
             charge <- iso[which(iso$feature == feat),"charge"]
         }
         charge
-    })
+    }, numeric(1))
     df.ret <- cbind(df.clique, chargeV)
     colnames(df.ret) <- c("mz","isotope","feature","charge")
     return(df.ret)
 }
 
+addIsoAnnotation <- function(isotopes, annotation, anclique) {
+    newannotation <- annotation
+    if( sum(is.na(unlist(isotopes))) !=
+        length(unlist(isotopes)) ) {
+        isoAn <- isotopeAnnotation(newannotation, anclique)
+        newannotation <- rbind(newannotation, isoAn)
+    }
+    return(newannotation)
+}
+
+
+transformAnnotation <- function(peaklist, annotation) {
+    ## function to write the annotation into the peaklist
+    ## of the anclique object, and transforming into a
+    ## character
+    newpeaklist <- peaklist
+    newannotation <- annotation
+    newannotation <- newannotation[order(newannotation$feature),]
+    newannotation <- newannotation[,-1]
+    newpeaklist <- cbind(newpeaklist, newannotation)
+    ## transform annotation in character
+
+    newpeaklist$an1 = as.character(newpeaklist$an1)
+    newpeaklist$an1[newpeaklist$an1 == "NA"] <- NA
+    newpeaklist$an2 = as.character(newpeaklist$an2)
+    newpeaklist$an2[newpeaklist$an2 == "NA"] <- NA
+    newpeaklist$an3 = as.character(newpeaklist$an3)
+    newpeaklist$an3[newpeaklist$an3 == "NA"] <- NA
+    newpeaklist$an4 = as.character(newpeaklist$an4)
+    newpeaklist$an4[newpeaklist$an4 == "NA"] <- NA
+    newpeaklist$an5 = as.character(newpeaklist$an5)
+    newpeaklist$an5[newpeaklist$an5 == "NA"] <- NA
+    ## transform masses that are 0 in na
+    newpeaklist$mass1[newpeaklist$mass1 == 0] <- NA
+    newpeaklist$mass2[newpeaklist$mass2 == 0] <- NA
+    newpeaklist$mass3[newpeaklist$mass3 == 0] <- NA
+    newpeaklist$mass4[newpeaklist$mass4 == 0] <- NA
+    newpeaklist$mass5[newpeaklist$mass5 == 0] <- NA
+    return(newpeaklist)
+}
+
+
 #' @export
 #' @title Annotate adducts and fragments
 #'
@@ -130,17 +172,25 @@ getIsoCharge <- function(df.clique, iso) {
 #' have a relative mass difference smaller than 'filter' and the same
 #' features and adducts, drop the neutral mass with less adducts
 #' @param emptyS Score given to non annotated features. If you use your own
-#' 'adinfo', do not set 'emptyS' bigger than any adduct frequency in your list.
-#' @details Reported scores do not always refer to the entire clique group.
+#' 'adinfo', do not set 'emptyS' bigger than any adduct log frequency in
+#' your list.
+#' @param normalizeScore If 'TRUE', the reported score is normalized and scaled.
+#' Normalized score goes from 0, when it means that the raw score
+#' is close to the minimum score (all features with
+#' empty annotations), up to 100, which is the score value of the theoretical
+#' maximum annotation (all the adducts of the list with the
+#' minimum number of neutral masses).
+#' @details The default 'adinfo' lists are 'positive.adinfo' and
+#' 'negative.adinfo'. For use load them with 'data(positive.adinfo)' or
+#' data(negative.adinfo) commands.
+#' Reported scores do not always refer to the entire clique group.
 #' There might be features whose annotation is independent
 #' from other features of the clique group. This occurs when there are 
 #' no neutral masses with adducts in both groups of features.
 #' Therefore, the clique
 #' group is divided in non overlapping regions, called annotation groups.
-#' Scores report for these annotation groups. The score is intended to compare
-#' annotations within the same group, but do not compare scores of different
-#' groups because the score is smaller if the number of features in the group
-#' is bigger.
+#' Scores report for these annotation groups. To compare scores between
+#' different groups use 'normalizeScore' = TRUE.
 #'
 #' If clique groups have a lot of features, there are
 #' many combinations of neutral masses and adducts. This could
@@ -153,79 +203,54 @@ getIsoCharge <- function(df.clique, iso) {
 #' @return An 'anClique' object with annotation columns added
 #' to the peaklist
 #' @examples
-#' library(cliqueMS)
-#' summary(ex.cliqueGroups)
+#' data(ex.cliqueGroups)
+#' show(ex.cliqueGroups)
 #' ex.isoAn <- getIsotopes(ex.cliqueGroups)
-#' summary(ex.isoAn)
+#' show(ex.isoAn)
+#' data(positive.adinfo)
 #' ex.adductAn <- getAnnotation(ex.isoAn, positive.adinfo, 'positive')
-#' @seealso \code{\link{getCliques}}
-#' \code{\link{getIsotopes}}
 getAnnotation <- function(anclique, adinfo, polarity, topmasstotal = 10, 
-                          topmassf = 1, sizeanG = 20,
-                          ppm = 10, filter = 1e-4, emptyS = 1e-6) {
+    topmassf = 1, sizeanG = 20, ppm = 10, filter = 1e-4, emptyS = -6,
+    normalizeScore = TRUE) {
     if( (polarity != "positive")&&(polarity != "negative") ) {
         stop("Polarity has to be 'positive' or 'negative'")
     }
     orderadinfo = checkadinfo(adinfo, polarity)
-    if(anclique$anFound == TRUE) {
+    if(anclique@anFound == TRUE) {
         warning("Annotation has already been computed for this object")
     }
-    if(anclique$isoFound == FALSE) {
-        warning(paste("Isotopes have not been annotated\n",
-                      "This could lead to some errors in adduct",
-                      "annotation\n"))
+    if(anclique@isoFound == FALSE) {
+        warning("Isotopes have not been annotated.
+            This could lead to some errors in adduct annotation\n")
     }
-    if(anclique$cliquesFound == FALSE) {
-        warning(paste("Cliques have not been computed\n",
-                      "This could lead to long computing times",
-                      "for adduct annotation\n"))
+    if(anclique@cliquesFound == FALSE) {
+        warning("Cliques have not been computed.
+            This could lead to long computing times
+            for adduct annotation")
     }
-    cat("Computing annotation\n")
-    # Compute annotation for each clique
+    message("Computing annotation")
+    ## Compute annotation for each clique
     ppm = 1e-6*ppm
-    anList <- lapply(anclique$cliques, function(x) {
+    anList <- lapply(anclique@cliques, function(x) {
         df.clique <- as.data.frame(
-            cbind(anclique$peaklist[x, c("mz","isotope")],x)
+            cbind(anclique@peaklist[x, c("mz","isotope")],x)
         )
         colnames(df.clique) <- c("mz","isotope","feature")
         df.clique <- df.clique[grep("^M0", df.clique$isotope),]
-        # set charge to isotopic features
-        df.clique <- getIsoCharge(df.clique, anclique$isotope)
-        df.clique <- df.clique[order(df.clique$mz, decreasing = F),]
+        ## set charge to isotopic features
+        df.clique <- getIsoCharge(df.clique, anclique@isotopes)
+        df.clique <- df.clique[order(df.clique$mz, decreasing = FALSE),]
         annotation <- returnAnnotation(df.clique, orderadinfo,
-                                       topmassf, topmasstotal,
-                                       sizeanG, ppm,
-                                       filter, emptyS)
+            topmassf, topmasstotal,
+            sizeanG, ppm,
+            filter, emptyS, normalizeScore)
     })
     df.annotation <- do.call(rbind, anList)
-    cat("Annotation computed, updating peaklist\n")
-    # Now add the annotation for isotopes if there are isotopes
-    if( sum(is.na(unlist(anclique$isotopes))) !=
-            length(unlist(anclique$isotopes)) ) {
-        isoAn <- isotopeAnnotation(df.annotation, anclique)
-        df.annotation <- rbind(df.annotation, isoAn)
-    }
-    df.annotation <- df.annotation[order(df.annotation$feature),]
-    df.annotation <- df.annotation[,-1]
-    anclique$peaklist <- cbind(anclique$peaklist, df.annotation)
-    # transform annotation in character
-    anclique$peaklist$an1 = as.character(anclique$peaklist$an1)
-    anclique$peaklist$an1[anclique$peaklist$an1 == "NA"] <- NA
-    anclique$peaklist$an2 = as.character(anclique$peaklist$an2)
-    anclique$peaklist$an2[anclique$peaklist$an2 == "NA"] <- NA
-    anclique$peaklist$an3 = as.character(anclique$peaklist$an3)
-    anclique$peaklist$an3[anclique$peaklist$an3 == "NA"] <- NA
-    anclique$peaklist$an4 = as.character(anclique$peaklist$an4)
-    anclique$peaklist$an4[anclique$peaklist$an4 == "NA"] <- NA
-    anclique$peaklist$an5 = as.character(anclique$peaklist$an5)
-    anclique$peaklist$an5[anclique$peaklist$an5 == "NA"] <- NA
-    # transform masses that are 0 in na
-    anclique$peaklist$mass1[anclique$peaklist$mass1 == 0] <- NA
-    anclique$peaklist$mass2[anclique$peaklist$mass2 == 0] <- NA
-    anclique$peaklist$mass3[anclique$peaklist$mass3 == 0] <- NA
-    anclique$peaklist$mass4[anclique$peaklist$mass4 == 0] <- NA
-    anclique$peaklist$mass5[anclique$peaklist$mass5 == 0] <- NA
-    # uptade object information
-    anclique$anFound <- TRUE
+    message("Annotation computed, updating peaklist")
+    df.annotation <- addIsoAnnotation(anclique@isotopes, df.annotation,
+        anclique)
+    anclique@peaklist <- transformAnnotation(anclique@peaklist, df.annotation)
+    ## update object information
+    anclique@anFound <- TRUE
     return(anclique)
 }
